@@ -10,11 +10,7 @@ import traceback
 try:
     import rdflib
 
-    if rdflib.__version__ < "6.0.0":
-        logger.error("RDFlib >= 6.0.0 is required. Please upgrade.")
-        import sys
-
-        sys.exit(1)
+    logger.info("Using RDFlib %s" % rdflib.__version__)
 except ImportError:
     logger.error("RDFlib is required. Please install it.")
     import sys
@@ -96,6 +92,8 @@ def parse_stmts_to_graph(stmts):
     data = N3_PROLOGUE
 
     for stmt in stmts:
+        if " " not in stmt:
+            raise KbServerError("invalid syntax for statement %s: it should be formed of 3 terms." % stmt)
         data += " %s . " % stmt
 
     try:
@@ -117,6 +115,8 @@ def parse_stmt(stmt):
         return list(Graph().parse(data=N3_PROLOGUE + "%s ." % stmt, format="n3"))[0]
     except rdflib.plugins.parsers.notation3.BadSyntax:
         raise KbServerError("invalid syntax for statement <%s>" % stmt)
+    except IndexError:
+        raise KbServerError("invalid syntax for statement <%s>" % stmt)
 
 
 # @memoize
@@ -667,13 +667,13 @@ class MinimalKB:
                 self.models[model].graph += subgraph  # TODO: lifespan
                 self.models[model].is_dirty = True
 
-        if policy["method"] == "retract":
+        elif policy["method"] == "retract":
             logger.info("Deleting from " + str(list(models)) + ":\n\t- " + parsed_stmts)
             for model in models:
                 self.models[model].graph -= subgraph
                 self.models[model].is_dirty = True
 
-        if policy["method"] in ["update", "safe_update", "revision"]:
+        elif policy["method"] in ["update", "safe_update", "revision"]:
 
             lifespan = policy.get("lifespan", 0)
 
@@ -692,6 +692,8 @@ class MinimalKB:
                     else:
                         self.models[model].graph.add((s, p, o))  # TODO: lifespan
                 self.models[model].is_dirty = True
+        else:
+            raise KbServerError("Unknown method in revise: %s" % policy["method"])
 
         self.onupdate()
 
@@ -751,11 +753,14 @@ class MinimalKB:
 
         model = list(models)[0]
 
-        sparql_res = self._sparql(model, query)
+        sparql_res, query = self._sparql(model, query)
 
         import json
 
-        return json.loads(sparql_res.serialize(format="json"))
+        res = json.loads(sparql_res.serialize(format="json"))
+        res["query"] = query
+
+        return res
 
     @api
     def find(self, patterns, vars=None, models=None):
@@ -833,7 +838,7 @@ class MinimalKB:
                 q += "%s .\n" % p
             q += "}"
 
-            sparql_res = self._sparql(model, q)
+            sparql_res, _ = self._sparql(model, q)
 
             res += [
                 dict(zip(vars_naked, r))
@@ -908,7 +913,11 @@ class MinimalKB:
 
         logger.debug("Executing SPARQL query in model: %s\n%s" % (model, q))
 
-        return self.models[model].materialized_graph.query(q)
+        import pyparsing
+        try:
+            return self.models[model].materialized_graph.query(q), q
+        except pyparsing.ParseException:
+            raise KbServerError("Syntax error while parsing SPARQL query:\n%s" % q)
 
     def named_variables(self, vars):
         """
